# frozen_string_literal: true

# CarrierWave Secure - Licensed under LGPL-3.0

require 'active_support/concern'

module CarrierWave
  module Uploader
    # Secure uploader factory
    #
    # Uploader instantiation:
    #
    # ```ruby
    # uploader = CarrierWave::Uploader::Secure[CarrierWave::Uploader::Base].new
    # ```
    module Secure
      # Secure uploader
      #
      # Extends CarrierWave::Uploader::Base by adding encryption/decryption operations before
      # uploading and after dowloading files.
      module Base
        extend ActiveSupport::Concern

        def store!(new_file = file)
          sanitized = CarrierWave::SanitizedFile.new(new_file)
          encrypted = CarrierWave::Secure::EncryptedFile.new(sanitized)
          super(encrypted)
          @file = CarrierWave::Secure::DecryptedFile.new(encrypted)
        end

        def retrieve_from_store!(identifier)
          super
          @file = CarrierWave::Secure::DecryptedFile.new(file) if file
        end
      end

      def self.[](extended_class)
        Class.new(extended_class) { include CarrierWave::Uploader::Secure::Base }
      end
    end
  end
end
